package middleware

import (
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

type SavedContext struct {
	ExtraLogFields []zap.Field
	AuditLogFields []zap.Field
	Errs           []string
}

// Write the gin contxt values we want to save into the SavedGinContext map stored higher up in the chain.
func (mwf RouteMiddlewareFactory) TranslateGinContext() gin.HandlerFunc {
	return func(c *gin.Context) {
		defer func() {
			ginContext := c.Request.Context()
			savedContext := ginContext.Value(SavedGinContext).(*SavedContext)
			if savedContext == nil {
				return
			}
			errs := c.Errors.Errors()
			if len(errs) > 0 {
				savedContext.Errs = errs
			}
			extraLogFields := c.Value(string(ExtraLogFields))
			if extraLogFields != nil {
				if fields, ok := extraLogFields.([]zap.Field); ok {
					savedContext.ExtraLogFields = fields
				}
			}
			auditLogFields := c.Value(string(AuditLogFields))
			if auditLogFields != nil {
				if fields, ok := auditLogFields.([]zap.Field); ok {
					savedContext.AuditLogFields = fields
				}
			}
		}()
		c.Next()
	}
}
