// Copyright (c) 2023 VMware Carbon Black, all rights reserved

// Package models -- thos package contains the various data structures used to configure the hint generator
package models

import (
	"testing"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/enums/devicechangetype"
	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/enums/devicegroupchangetype"
	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/enums/orgchangetype"
	"gitlab.bit9.local/psc/ems/hinting/pkg/enums/deploymenttype"
	"gitlab.bit9.local/psc/ems/hinting/pkg/enums/devicetype"
)

var _ = Describe("Enum functions testing", func() {
	It("Ensure we handle unknown deployment types", func() {
		check := deploymenttype.FromString("GOOBER")
		Expect(check).ToNot(BeEquivalentTo(deploymenttype.UNKNOWN)) // should be converted to "deploymenttype.GOOBER"
	})

	It("Ensure we handle empty deployment types", func() {
		check := deploymenttype.FromString("")
		Expect(check).To(BeEquivalentTo(deploymenttype.UNKNOWN))
	})

	It("Ensure we trap trailing/ending spaces in deployment types", func() {
		check := deploymenttype.FromString("  ENDPOINT  ")
		Expect(check).To(BeEquivalentTo(deploymenttype.ENDPOINT))
	})

	It("Ensure we handle casing in deployment types", func() {
		check := deploymenttype.FromString("endpoint")
		Expect(check).To(BeEquivalentTo(deploymenttype.ENDPOINT))
	})

	// ----------

	It("Ensure we handle unknown device types", func() {
		check := devicetype.FromString("GOOBER")
		Expect(check).ToNot(BeEquivalentTo(deploymenttype.UNKNOWN)) // should be converted to "devicetype.GOOBER"
	})

	It("Ensure we handle empty device types", func() {
		check := devicetype.FromString("")
		Expect(check).To(BeEquivalentTo(devicetype.UNKNOWN))
	})

	It("Ensure we trap trailing/ending spaces in device types", func() {
		check := devicetype.FromString("  WINDOWS  ")
		Expect(check).To(BeEquivalentTo(devicetype.WINDOWS))
	})

	It("Ensure we handle casing in device types", func() {
		check := devicetype.FromString("Windows")
		Expect(check).To(BeEquivalentTo(devicetype.WINDOWS))
	})

	// ----------

	It("Ensure we properly stringify DeviceChangeType", func() {
		check := devicechangetype.DEV_GROUP_CHANGE.String() // first
		Expect(check).To(BeEquivalentTo("DEV_GROUP_CHANGE"))
		check = devicechangetype.DEV_REREGISTER.String() // last
		Expect(check).To(BeEquivalentTo("DEV_REREGISTER"))
	})

	It("Ensure we properly stringify DeviceGroupChangeType", func() {
		check := devicegroupchangetype.DG_POLICY_CHANGE.String() // first
		Expect(check).To(BeEquivalentTo("DG_POLICY_CHANGE"))
		check = devicegroupchangetype.DG_LIVE_QUERY_CHANGE.String() // last
		Expect(check).To(BeEquivalentTo("DG_LIVE_QUERY_CHANGE"))
	})

	It("Ensure we properly stringify OrgChangeType", func() {
		check := orgchangetype.ORG_NET_CFG_CHANGE.String() // first
		Expect(check).To(BeEquivalentTo("ORG_NET_CFG_CHANGE"))
		check = orgchangetype.ORG_BLADE_ACTION_CHANGE.String() // last
		Expect(check).To(BeEquivalentTo("ORG_BLADE_ACTION_CHANGE"))
	})
})

func TestEnums(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "All Enum Suite")
}
