package models

import (
	"strings"
	"time"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/types"
	"gitlab.bit9.local/psc/ems/hinting/internal/metrics"
	"gitlab.bit9.local/psc/ems/hinting/pkg/enums/devicetype"
)

var _ = Describe("Postgres Organization Model tests", func() {
	It("results in nil dimensions with 'empty' env", func() {
		d := &Environment{}

		m := d.GetMetricDimensions()

		Expect(m).To(BeNil())
	})

	It("results in nil even if dimensions if name is set", func() {
		d := &Environment{
			Name: "local",
		}

		m := d.GetMetricDimensions()

		Expect(m).To(BeNil())
	})

	It("properly sets dimensions with a single hint", func() {
		d := &Environment{
			Name:            "local",
			CertServiceTime: func(t time.Time) *time.Time { return &t }(time.Now()),
		}

		m := *d.GetMetricDimensions()

		Expect(m).ToNot(BeNil())

		Expect(m[metrics.DimensionEnvName]).To(Equal("local"))
		Expect(strings.Contains(m["env_hints"], "env_CertServiceTime")).To(BeTrue())
		Expect(len(strings.Split(m["env_hints"], ","))).To(Equal(1))
		Expect(len(m)).To(Equal(2))
	})

	It("properly sets dimensions with a multiple hints", func() {
		d := &Environment{
			Name:                    "local",
			CertServiceTime:         func(t time.Time) *time.Time { return &t }(time.Now()),
			PolicyServiceUpdateTime: func(t time.Time) *time.Time { return &t }(time.Now()),
		}

		m := *d.GetMetricDimensions()

		Expect(m).ToNot(BeNil())

		Expect(m[metrics.DimensionEnvName]).To(Equal("local"))
		Expect(strings.Contains(m["env_hints"], "env_CertServiceTime")).To(BeTrue())
		Expect(strings.Contains(m["env_hints"], "env_PolicyServiceUpdateTime")).To(BeTrue())
		Expect(len(strings.Split(m["env_hints"], ","))).To(Equal(2))
		Expect(len(m)).To(Equal(2))
	})

	It("properly sets dimensions with canary", func() {
		d := &Environment{
			Name: "local",
			CanaryPayloadTimestamps: types.DeviceTypeToTimeMap{
				devicetype.WINDOWS: time.Now(),
				devicetype.MAC:     time.Now(),
			},
		}

		m := *d.GetMetricDimensions()

		Expect(m).ToNot(BeNil())

		Expect(m[metrics.DimensionEnvName]).To(Equal("local"))
		Expect(strings.Contains(m["env_canary_payload"], string(devicetype.WINDOWS))).To(BeTrue())
		Expect(strings.Contains(m["env_canary_payload"], string(devicetype.MAC))).To(BeTrue())
		Expect(len(strings.Split(m["env_canary_payload"], ","))).To(Equal(2))
		Expect(len(m)).To(Equal(2))
	})
})
