// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package models

import (
	"fmt"

	"gitlab.bit9.local/psc/ems/hinting/internal/metrics"
	"go.uber.org/zap/zapcore"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

type Model interface {
	CacheKey() string
	GetOnConflictClause() clause.OnConflict
	zapcore.ObjectMarshaler
	GetMetricDimensions() *metrics.Dimensions
}

type ModelWithOrgKey interface {
	Model
	getOrgKey() string
}

// ModelPointer -- This interface allows someone to specify a type-parameter which fufills Model and MUST BE pointer
//   - Reference link: https://go.googlesource.com/proposal/+/HEAD/design/43651-type-parameters.md
type ModelPointer[T any] interface {
	Model
	*T
}

type APIModel[T Model] interface {
	ConvertToDBObject() (T, error)
	GetURIPart() URIPart[T]
	GetBodyPart() any
}

type APIModelPointer[T any, P Model] interface {
	APIModel[P]
	*T
}

type URIPart[T Model] interface {
	ConvertToDBObject() (T, error)
}

func getGreatestExpr(tableName string, columns ...string) map[string]any {
	temp := make(map[string]any, len(columns))
	for _, column := range columns {
		temp[column] = gorm.Expr(fmt.Sprintf("GREATEST(%s.%s, excluded.%s)", tableName, column, column))
	}
	return temp
}
