// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package models

import (
	"encoding/json"
	"fmt"
	"testing"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/types"
	"gitlab.bit9.local/psc/ems/hinting/pkg/enums/deploymenttype"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
)

func runTest(name string, source, target Model) {
	originalModelStr := fmt.Sprintf("%v", source)

	fmt.Printf("Checking %s: %s", name, originalModelStr)
	modelMarshalBytes, err := json.Marshal(source)
	Expect(err).To(BeNil())

	err = json.Unmarshal(modelMarshalBytes, target)
	Expect(err).To(BeNil())

	unmarshalledModelStr := fmt.Sprintf("%v", target)
	Expect(unmarshalledModelStr).To(Equal(originalModelStr))
}

var _ = Describe("Device Model marshal/unmarshal tests", func() {
	now := time.Date(2022, 12, 19, 12, 30, 25, 0, time.Now().Location())

	// ----- Begin marshal/unmarshal tests --------------------

	It("Check Device transparency", func() {
		runTest("Device",
			NewDeviceTimeVaries(1, "orgKey", &now),
			new(Device),
		)
	})

	It("Check DeviceGroup transparency", func() {
		runTest("Device Group",
			NewDeviceGroup(1337, "orgKey", &now, &now),
			new(DeviceGroup),
		)
	})
	It("Check Org transparency", func() {
		runTest("Org",
			NewOrg("enki01", true, types.NewChangesetAtTime(now),
				types.NewDeploymentTypeChangesAtTime(deploymenttype.AWS, now), types.NewAllDeviceCanaryPayLoadSetAtTime(now)),
			new(Organization),
		)
	})
	It("Check Env transparency", func() {
		runTest("Env",
			NewEnvWithCanaryTimes("enki01", &now, &now, now),
			new(Environment),
		)
	})
})

func TestApiModels(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "All API model suite")
}
